# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module to store SunGridEngine cluster management platform code"""

__author__ = "Felix Simkovic"
__date__ = "24 May 2017"
__version__ = "0.2"

import logging
import os
import re

from pyjob import cexec
from pyjob.platform.platform import ClusterPlatform

logger = logging.getLogger(__name__)


class SunGridEngine(ClusterPlatform):
    """Object to handle the Sun Grid Engine (SGE) management platform"""

    TASK_ID = "SGE_TASK_ID"

    @staticmethod
    def alt(jobid, priority=None):
        """Alter a job in the SGE queue
        
        Parameters
        ----------
        jobid : int
           The job id to remove
        priority : int, optional
           The priority level of the job

        Notes
        -----
        This function is currently still under development does not provide
        the full range of ``qalter`` flags.

        Todo
        ----
        * Add more functionality
        * Add better debug message to include changed options

        """
        cmd = ["qalter"]
        if priority:
            cmd += ["-p", str(priority)]
        cmd += [str(jobid)]
        cexec(cmd)
        logger.debug("Altered parameters for job %d in the queue", jobid)

    @staticmethod
    def hold(jobid):
        """Hold a job in the SGE queue

        Parameters
        ----------
        jobid : int
           The job id to remove

        """
        cexec(["qhold", str(jobid)])
        logger.debug("Holding back job %d from the queue", jobid)

    @staticmethod
    def kill(jobid):
        """Remove a job from the SGE queue
        
        Parameters
        ----------
        jobid : int
           The job id to remove
        
        """
        cexec(["qdel", str(jobid)])
        logger.debug("Removed job %d from the queue", jobid)

    @staticmethod
    def rls(jobid):
        """Release a job from the SGE queue
        
        Parameters
        ----------
        jobid : int
           The job id to remove

        """
        cexec(["qrls", str(jobid)])
        logger.debug("Released job %d from the queue", jobid)

    @staticmethod
    def stat(jobid):
        """Obtain information about a job id

        Parameters
        ----------
        jobid : int
           The job id to remove

        Returns
        -------
        dict
           A dictionary with job specific data

        """
        stdout = cexec(["qstat", "-j", str(jobid)], permit_nonzero=True)
        data = {}
        line_split = re.compile(":\s+")
        for line in stdout.split(os.linesep):
            line = line.strip()
            if 'jobs do not exist' in line:
                return data
            if not line or "=" * 30 in line:
                continue
            else:
                kv = line_split.split(line, 1)
                if len(kv) == 2:
                    data[kv[0]] = kv[1]
        return data

    @staticmethod
    def sub(command, array=None, deps=None, directory=None, hold=False, log=None, name=None, pe_opts=None,
            priority=None, queue=None, runtime=None, shell=None, threads=None, *args, **kwargs):
        """Submit a job to the SGE queue

        Parameters
        ----------
        command : list
           A list with the final command
        array : list, optional
           A list of array instructions in form of [min, max, (step)].
           ``step`` is optional.
        deps : list, optional
           A list of dependency job ids
        directory : str, optional
           A path to a directory to run the job in
        hold : bool, optional
           Submit but __hold__ the job
        log : str, optional
           The path to a logfile for stdout
        name : str, optional
           The name of the job
        pe_opts : list, optional
           Job-specific keywords
        priority : int, optional
           The priority level of the job
        queue : str, optional
           The queue to submit the job to
        runtime : int, optional
           The maximum runtime of the job in seconds
        shell : str, optional
           The absolute path to the shell to run the job in
        threads : int, optional
           The maximum number of threads available to a job

        """
        cmd = ["qsub", "-cwd", "-V", "-w", "e", "-j", "y"]
        if array and len(array) == 3:
            cmd += ["-t", "{0}-{1}".format(array[0], array[1]), "-tc", str(array[2])]
        elif array and len(array) == 2:
            cmd += ["-t", "{0}-{1}".format(array[0], array[1])]
        if deps:
            cmd += ["-hold_jid", "{0}".format(",".join(map(str, deps)))]
        if hold:
            cmd += ["-h"]
        if log:
            cmd += ["-o", log]
        if name:
            cmd += ["-N", name]
        if pe_opts:
            cmd += ["-pe"] + pe_opts.split()
        if priority:
            cmd += ["-p", str(priority)]
        if queue:
            cmd += ["-q", queue]
        if runtime:
            cmd += ["-l", "h_rt={0}".format(runtime)]
        if shell:
            cmd += ["-S", shell]
        if threads:
            cmd += ["-pe mpi", str(threads)]
        cmd += command if isinstance(command, list) else [command]
        stdout = cexec(cmd, directory=directory)
        jobid = int(stdout.split()[2].split(".")[0]) if array else int(stdout.split()[2])
        logger.debug("Job %d successfully submitted to the SGE queue", jobid)
        return jobid



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module to store a stopwatch class"""

__author__ = "Felix Simkovic"
__date__ = "02 Jun 2017"
__version__ = "1.0"

import datetime
import logging
import time

logger = logging.getLogger(__name__)


class _Time(object):
    """Generic time class"""
    def __init__(self, index):
        """Instantiate a new :obj:`Lap`"""
        self.index = index
        self._start_time = 0.0
        self._end_time = 0.0

    def __repr__(self):
        return "{0}(index={1} time={2}s)".format(
            self.__class__.__name__, self.index, self.time)

    def __add__(self, other):
        """Add the lap times"""
        return self.time + other.time

    def __sub__(self, other):
        """Subtract the lap times"""
        return self.time - other.time

    @property
    def time(self):
        """Time"""
        return self._end_time - self._start_time

    @property
    def time_pretty(self):
        """Convert (seconds) to (days, hours, minutes, seconds)"""
        d = datetime.datetime(1970, 1, 1) + datetime.timedelta(seconds=self.time)
        # Leave -1 in day as we start on the first day of the year
        return d.day - 1, d.hour, d.minute, d.second


class _Lap(_Time):
    """Lap time"""
    def __init__(self, index):
        """Instantiate a new :obj:`Lap`"""
        super(_Lap, self).__init__(index)


class _Interval(_Time):
    """Interval time"""

    def __init__(self, index):
        """Instantiate a new :obj:`Interval`"""
        super(_Interval, self).__init__(index)

        self._laps = []
        self._locked = False
        self._running = False

        self._ilap = 0

    def __getitem__(self, id):
        """Slice the intervals"""
        return self._laps[id]

    @property
    def average(self):
        """The average lap time in ms"""
        if len(self._laps) < 1:
            logger.critical("No laps taken!")
            return 0.0
        lap_times = [lap.time for lap in self._laps]
        return sum(lap_times) / float(len(lap_times))

    @property
    def lap(self):
        """Take a lap snapshot"""
        if self._locked:
            logger.critical("Cannot add a lap, interval finished!")
            return None
        elif not self._running:
            logger.critical("Cannot add a lap, interval not running!")
            return None

        self._ilap += 1
        lap = _Lap(self._ilap)
        lap._start_time = self._start_time + sum([l.time for l in self._laps])
        lap._end_time = time.time()
        self._laps += [lap]

        return lap

    @property
    def laps(self):
        """The laps"""
        return self._laps

    @property
    def nlaps(self):
        """Number of laps"""
        return len(self._laps)

    @property
    def time(self):
        """Total runtime"""
        if self._running:
            return int(round(time.time() - self._start_time))
        else:
            return int(round(self._end_time - self._start_time))

    def start(self):
        """Start the interval"""
        if self._running:
            logger.warning("Interval is running ...")
        elif self._locked:
            logger.warning("Interval is locked ...")
        else:
            logger.debug("Starting new interval ...")
            self._start_time = time.time()
            self._running = True

    def stop(self):
        """Stop the interval"""
        if self._running:
            logger.debug("Stopping interval ...")
            self._end_time = time.time()
            self._running = False
            self._locked = True
        else:
            logger.warning("Interval not running!")


class StopWatch(_Time):
    """Stopwatch class"""

    def __init__(self):
        """Instantiate a new :obj:`StopWatch`"""
        super(StopWatch, self).__init__(1)
        self.reset()

    def __getitem__(self, id):
        """Slice the intervals"""
        return self._intervals[id]

    def __repr__(self):
        return "{0}(time={1}s intervals={2})".format(
            self.__class__.__name__, self.time, len(self._intervals)
        )

    @property
    def intervals(self):
        """The intervals taken"""
        return self._intervals

    @property
    def lap(self):
        """Take a lap snapshot"""
        if len(self._intervals) > 0 and self._intervals[-1]._running:
            return self._intervals[-1].lap
        else:
            logger.critical("Cannot add a lap, stopwatch not running!")
            return None

    @property
    def nintervals(self):
        """Number of intervals"""
        return len(self._intervals)

    @property
    def running(self):
        """Stopwatch status"""
        return len(self._intervals) > 0 and self._intervals[-1]._running 

    @property
    def time(self):
        """Time in seconds"""
        return sum([interval.time for interval in self._intervals])

    @property
    def time_pretty(self):
        """Convert (seconds) to (days, hours, minutes, seconds)"""
        d = datetime.datetime(1970, 1, 1) + datetime.timedelta(seconds=self.time)
        # Leave -1 in day as we start on the first day of the year
        return d.day - 1, d.hour, d.minute, d.second

    def reset(self):
        """Reset the timer"""
        self._intervals = []
        self._running = False
        self._iinterval = 0

    def start(self):
        """Start the interval"""
        if len(self._intervals) > 0 and self._intervals[-1]._running:
            logger.warning("Stopwatch already running!")
        else:
            logger.debug("Starting stopwatch ...")
            self._iinterval += 1
            interval = _Interval(self._iinterval)
            interval.start()
            self._intervals += [interval]
        return interval

    def stop(self):
        """Stop the interval"""
        if len(self._intervals) > 0 and self._intervals[-1]._running:
            logger.debug("Stopping stopwatch ...")
            self._intervals[-1].stop()
        else:
            logger.warning("Stopwatch not running!")
        return self._intervals[-1]



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Test module for pyjob.platform"""

__author__ = "Felix Simkovic"
__date__ = "03 Jun 2017"

import os
import unittest

from pyjob.platform import prep_array_script


class Test(unittest.TestCase):

    def test_prep_array_scripts_1(self):
        scripts = [os.path.join(os.getcwd(), "script1.sh"), os.path.join(os.getcwd(), "script2.sh")]
        array_script, array_jobs = prep_array_script(scripts, os.getcwd(), "SGE_TASK_ID")
        with open(array_jobs) as f_in:
            array_jobs_content = [l.strip() for l in f_in.readlines()]
        self.assertEqual(scripts, array_jobs_content)
        with open(array_script) as f_in:
            array_script_content = [l.strip() for l in f_in.readlines()]
        self.assertEqual([
            "#!/bin/sh",
            'script=$(awk "NR==$SGE_TASK_ID" ' + array_jobs + ')',
            "log=$(echo $script | sed 's/.sh/.log/')",
            "$script > $log"
        ], array_script_content)
        for f in [array_script, array_jobs]:
            os.unlink(f)

    def test_prep_array_scripts_2(self):
        scripts = [os.path.join(os.getcwd(), "script1.sh"), os.path.join(os.getcwd(), "script2.sh")]
        array_script, array_jobs = prep_array_script(scripts, os.getcwd(), "LSB_JOBINDEX")
        with open(array_jobs) as f_in:
            array_jobs_content = [l.strip() for l in f_in.readlines()]
        self.assertEqual(scripts, array_jobs_content)
        with open(array_script) as f_in:
            array_script_content = [l.strip() for l in f_in.readlines()]
        self.assertEqual([
            "#!/bin/sh",
            'script=$(awk "NR==$LSB_JOBINDEX" ' + array_jobs + ')',
            "log=$(echo $script | sed 's/.sh/.log/')",
            "$script > $log"
        ], array_script_content)
        for f in [array_script, array_jobs]:
            os.unlink(f)

    def test_prep_array_scripts_3(self):
        scripts = [os.path.join(os.getcwd(), "script1.sh"), os.path.join(os.getcwd(), "script2.sh")]
        array_script, array_jobs = prep_array_script(scripts, os.getcwd(), "RANDOM_TEXT")
        with open(array_jobs) as f_in:
            array_jobs_content = [l.strip() for l in f_in.readlines()]
        self.assertEqual(scripts, array_jobs_content)
        with open(array_script) as f_in:
            array_script_content = [l.strip() for l in f_in.readlines()]
        self.assertEqual([
            "#!/bin/sh",
            'script=$(awk "NR==$RANDOM_TEXT" ' + array_jobs + ')',
            "log=$(echo $script | sed 's/.sh/.log/')",
            "$script > $log"
        ], array_script_content)
        for f in [array_script, array_jobs]:
            os.unlink(f)

    def test_prep_array_scripts_4(self):
        scripts = [os.path.join(os.getcwd(), "script1.sh"), os.path.join(os.getcwd(), "script2.sh")]
        array_script, array_jobs = prep_array_script(scripts, "pyjob", "RANDOM_TEXT")
        self.assertEqual(os.path.abspath("pyjob"), os.path.dirname(array_script))
        self.assertEqual(os.path.abspath("pyjob"), os.path.dirname(array_jobs))
        with open(array_jobs) as f_in:
            array_jobs_content = [l.strip() for l in f_in.readlines()]
        self.assertEqual(scripts, array_jobs_content)
        with open(array_script) as f_in:
            array_script_content = [l.strip() for l in f_in.readlines()]
        self.assertEqual([
            "#!/bin/sh",
            'script=$(awk "NR==$RANDOM_TEXT" ' + array_jobs + ')',
            "log=$(echo $script | sed 's/.sh/.log/')",
            "$script > $log"
        ], array_script_content)
        for f in [array_script, array_jobs]:
            os.unlink(f)


if __name__ == "__main__":
    unittest.main(verbosity=2)




"""The version number of pyjob is recorded here"""

# Maintain sematantic versioning. Further information can
# be found here [http://semver.org/]
__version_info__ = (0, 1, 2)

# ======================================================
# Do __NOT__ change anything below here
__version__ = '.'.join(str(v) for v in __version_info__)



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module to store LoadSharingFacility cluster management platform code"""

__author__ = "Felix Simkovic"
__date__ = "30 May 2017"
__version__ = "0.1"

import logging
import os

from pyjob import cexec
from pyjob.platform.platform import ClusterPlatform

logger = logging.getLogger(__name__)


class LoadSharingFacility(ClusterPlatform):
    """Object to handle the Load Sharing Facility (LSF) management platform"""

    TASK_ID = "LSB_JOBINDEX"

    @staticmethod
    def hold(jobid):
        """Hold a job in the LSF queue

        Parameters
        ----------
        jobid : int
           The job id to remove

        """
        cexec(["bstop", str(jobid)])
        logger.debug("Holding back job %d from the queue", jobid)

    @staticmethod
    def kill(jobid):
        """Remove a job from the LSF queue
        
        Parameters
        ----------
        jobid : int
           The job id to remove

        Raises
        ------
        RuntimeError
           Execution exited with non-zero return code

        """
        stdout = cexec(["bkill", str(jobid)], permit_nonzero=True)
        # Large arrays can take time to be full deleted, give it a few moments
        if "is in progress" in stdout:
            stdout = cexec(["bkill", "-b", str(jobid)], permit_nonzero=True)
            import time
            time.sleep(10)
        if any(text in stdout for text in ["has already finished", "is being terminated", "is in progress"]):
            logger.debug("Removed job %d from the queue", jobid)
        else:
            msg = "Execution of '{0}' exited with non-zero return code: {1}".format("bkill " + str(jobid), stdout)
            raise RuntimeError(msg)

    @staticmethod
    def rls(jobid):
        """Release a job from the LSF queue

        Parameters
        ----------
        jobid : int
           The job id to remove

        """
        cexec(["bresume", str(jobid)])
        logger.debug("Released job %d from the queue", jobid)

    @staticmethod
    def stat(jobid):
        """Obtain information about a job id

        Parameters
        ----------
        jobid : int
           The job id to remove

        Returns
        -------
        dict
           A dictionary with job specific data

        Todo
        ----
        * Extract the correct information

        """
        stdout = cexec(["bjobs", "-l", str(jobid)])
        if "Done successfully" in stdout:
            return {}
        else:
            return {'job_number': jobid, 'status': "Running"}

    @staticmethod
    def sub(command, array=None, deps=None, directory=None, hold=False, log=None, name=None, priority=None,
            queue=None, runtime=None, shell=None, threads=None, *args, **kwargs):
        """Submit a job to the LSF queue

        Parameters
        ----------
        command : list
           A list with the final command
        array : list, optional
           A list of array instructions in form of [min, max, (step)].
           ``step`` is optional.
        deps : list, optional
           A list of dependency job ids
        directory : str, optional
           A path to a directory to run the job in
        hold : bool, optional
           Submit but __hold__ the job
        log : str, optional
           The path to a logfile for stdout
        name : str, optional
           The name of the job
        pe_opts : list, optional
           Job-specific keywords [Unused]
        priority : int, optional
           The priority level of the job
        queue : str, optional
           The queue to submit the job to
        runtime : int, optional
           The maximum runtime of the job in seconds
        shell : str, optional
           The absolute path to the shell to run the job in
        threads : int, optional
           The maximum number of threads available to a job

        """
        cmd = ["bsub", "-cwd", directory if directory else os.getcwd()]
        if array:
            name = "pyjob" if name is None else name
            if len(array) == 3:
                cmd += ["-J", "{0}[{1}-{2}%{3}]".format(name, array[0], array[1], array[2])]
            elif len(array) == 2:
                cmd += ["-J", "{0}[{1}-{2}]".format(name, array[0], array[1])]
            name = None             # Reset this!
        if deps:
            cmd += ["-w", " && ".join(["done(%s)" % dep for dep in map(str, deps)])]
        if hold:
            cmd += ["-H"]
        if log:
            cmd += ["-o", log]
        if name:
            cmd += ["-J", '"{0}"'.format(name)]
        if priority:
            cmd += ["-sp", str(priority)]
        if queue:
            cmd += ["-q", queue]
        if shell:
            cmd += ["-L", shell]
        if threads:
            cmd += ["-R", '"span[ptile={0}]"'.format(threads)]
        if runtime:
            cmd += ["-W", str(runtime)]
        command_f = command[0] if isinstance(command, list) else command
        stdout = cexec(cmd, stdin=open(command_f).read(), directory=directory)
        jobid = int(stdout.split()[1][1:-1])
        logger.debug("Job %d successfully submitted to the LSF queue", jobid)
        return jobid



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Testing facility for pyjob.platform.sge"""

__author__ = "Felix Simkovic"
__date__ = "10 May 2017"

import inspect
import os
import time
import unittest

from pyjob.misc import make_script
from pyjob.platform import prep_array_script
from pyjob.platform.sge import SunGridEngine

# Required for Python2.6 support
def skipUnless(condition):
    if condition:
        return lambda x: x
    else:
        return lambda x: None 


@skipUnless("SGE_ROOT" in os.environ)
class TestSunGridEngine(unittest.TestCase):

    # ================================================================================
    # SINGLE JOB SUBMISSIONS

    def test_alt_1(self):
        jobs = [make_script(["sleep 100"])]
        jobid = SunGridEngine.sub(jobs, hold=True, name=inspect.stack()[0][3], shell="/bin/sh")
        time.sleep(5)
        SunGridEngine.alt(jobid, priority=-1)
        data = SunGridEngine.stat(jobid)
        self.assertTrue(data)
        self.assertEqual(jobid, int(data['job_number']))
        self.assertEqual(-1, int(data['priority']))
        SunGridEngine.kill(jobid)
        for f in jobs:
            os.unlink(f)

    def test_kill_1(self):
        jobs = [make_script(["sleep 100"])]
        jobid = SunGridEngine.sub(jobs, hold=True, name=inspect.stack()[0][3], shell="/bin/sh")
        time.sleep(5)
        self.assertTrue(SunGridEngine.stat(jobid))
        SunGridEngine.kill(jobid)
        for f in jobs:
            os.unlink(f)

    def test_hold_1(self):
        jobs = [make_script(["sleep 100"])]
        jobid = SunGridEngine.sub(jobs, hold=False, name=inspect.stack()[0][3], shell="/bin/sh", log=os.devnull)
        time.sleep(5)
        SunGridEngine.hold(jobid)
        SunGridEngine.kill(jobid)
        map(os.unlink, jobs)

    def test_rls_1(self):
        jobs = [make_script(["touch", "pyjob_rls_test_1"])]
        jobid = SunGridEngine.sub(jobs, hold=True, name=inspect.stack()[0][3], shell="/bin/sh", log=os.devnull)
        time.sleep(5)
        SunGridEngine.rls(jobid)
        while SunGridEngine.stat(jobid):
            time.sleep(1)
        self.assertTrue(os.path.isfile('pyjob_rls_test_1'))
        os.unlink('pyjob_rls_test_1')
        for f in jobs:
            os.unlink(f)

    def test_stat_1(self):
        jobs = [make_script(["sleep 100"])]
        jobid = SunGridEngine.sub(jobs, hold=True, name=inspect.stack()[0][3], shell="/bin/sh")
        time.sleep(5)
        data = SunGridEngine.stat(jobid)
        self.assertTrue(data)
        self.assertEqual(jobid, int(data['job_number']))
        self.assertTrue('sge_o_shell' in data)
        self.assertTrue('sge_o_workdir' in data)
        self.assertTrue('sge_o_host' in data)
        SunGridEngine.kill(jobid)
        for f in jobs:
            os.unlink(f)

    def test_sub_1(self):
        jobs = [make_script(["sleep 1"])]
        jobid = SunGridEngine.sub(jobs, hold=True, name=inspect.stack()[0][3], shell="/bin/sh")
        time.sleep(5)
        self.assertTrue(SunGridEngine.stat(jobid))
        SunGridEngine.kill(jobid)
        for f in jobs:
            os.unlink(f)

    def test_sub_2(self):
        assert "PYJOB_ENV" not in os.environ
        os.environ["PYJOB_ENV"] = "pyjob_random"
        job = [make_script(["echo $PYJOB_ENV"])]
        log = job[0].replace(".sh", ".log")
        jobid = SunGridEngine.sub(job, log=log, name=inspect.stack()[0][3], shell="/bin/sh")
        time.sleep(5)
        while SunGridEngine.stat(jobid):
            time.sleep(1)
        self.assertTrue(os.path.isfile(log))
        self.assertTrue(open(log).read().strip(), "pyjob_random")
        for f in job + [log]:
            os.unlink(f)

    # ================================================================================
    # ARRAY JOB SUBMISSIONS

    def test_kill_2(self):
        jobs = [make_script(["sleep 100"]) for _ in range(5)]
        array_script, array_jobs = prep_array_script(jobs, os.getcwd(), SunGridEngine.TASK_ID)
        jobid = SunGridEngine.sub(array_script, array=[1, 5], hold=True,
                                  name=inspect.stack()[0][3], shell="/bin/sh")
        time.sleep(5)
        self.assertTrue(SunGridEngine.stat(jobid))
        SunGridEngine.kill(jobid)
        for f in jobs + [array_script, array_jobs]:
            os.unlink(f)

    def test_stat_2(self):
        jobs = [make_script(["sleep 100"]) for _ in range(5)]
        array_script, array_jobs = prep_array_script(jobs, os.getcwd(), SunGridEngine.TASK_ID)
        jobid = SunGridEngine.sub(array_script, array=[1, 5], hold=True, name=inspect.stack()[0][3], shell="/bin/sh")
        time.sleep(5)
        data = SunGridEngine.stat(jobid)
        SunGridEngine.kill(jobid)
        self.assertTrue(data)
        self.assertEqual(jobid, int(data['job_number']))
        self.assertTrue('sge_o_shell' in data)
        self.assertTrue('sge_o_workdir' in data)
        self.assertTrue('sge_o_host' in data)
        self.assertTrue('job-array tasks' in data)
        self.assertEqual("1-5:1", data['job-array tasks'].strip())
        for f in jobs + [array_script, array_jobs]:
            os.unlink(f)

    def test_sub_3(self):
        jobs = [make_script(["sleep 1"]) for _ in range(5)]
        array_script, array_jobs = prep_array_script(jobs, os.getcwd(), SunGridEngine.TASK_ID)
        jobid = SunGridEngine.sub(array_script, array=[1, 5], hold=True,
                                  name=inspect.stack()[0][3], shell="/bin/sh")
        time.sleep(5)
        self.assertTrue(SunGridEngine.stat(jobid))
        SunGridEngine.kill(jobid)
        for f in jobs + [array_script, array_jobs]:
            os.unlink(f)

    def test_sub_4(self):
        directory = os.getcwd()
        jobs = [make_script([["sleep 5"], ['echo "file {0}"'.format(i)]], directory=directory) 
                for i in range(5)]
        array_script, array_jobs = prep_array_script(jobs, directory, SunGridEngine.TASK_ID)
        jobid = SunGridEngine.sub(array_script, array=[1, 5], log=os.devnull,
                                  name=inspect.stack()[0][3], shell="/bin/sh")
        while SunGridEngine.stat(jobid):
            time.sleep(1)
        for i, j in enumerate(jobs):
            f = j.replace(".sh", ".log")
            self.assertTrue(os.path.isfile(f))
            self.assertEqual("file {0}".format(i), open(f).read().strip())
            os.unlink(f)
        for f in jobs + [array_script, array_jobs]:
            os.unlink(f)

    def test_sub_5(self):
        directory = os.getcwd()
        jobs = [make_script(['echo "file {0}"'.format(i)], directory=directory) 
                for i in range(100)]
        array_script, array_jobs = prep_array_script(jobs, directory, SunGridEngine.TASK_ID)
        jobid = SunGridEngine.sub(array_script, array=[1, 100], log=os.devnull,
                                  name=inspect.stack()[0][3], shell="/bin/sh")
        while SunGridEngine.stat(jobid):
            time.sleep(1)
        for i, j in enumerate(jobs):
            f = j.replace(".sh", ".log")
            self.assertTrue(os.path.isfile(f))
            self.assertEqual("file {0}".format(i), open(f).read().strip())
            os.unlink(f)
        for f in jobs + [array_script, array_jobs]:
            os.unlink(f)

    def test_sub_6(self):
        jobs = [make_script(["echo $SGE_ROOT"], directory=os.getcwd()) for _ in range(2)]
        array_script, array_jobs = prep_array_script(jobs, os.getcwd(), SunGridEngine.TASK_ID)
        jobid = SunGridEngine.sub(array_script, array=[1, 2], log=os.devnull,
                                  name=inspect.stack()[0][3], shell="/bin/sh")
        while SunGridEngine.stat(jobid):
            time.sleep(1)
        for i, j in enumerate(jobs):
            f = j.replace(".sh", ".log")
            self.assertTrue(os.path.isfile(f))
            self.assertEqual(os.environ["SGE_ROOT"], open(f).read().strip())
            os.unlink(f)
        for f in jobs + [array_script, array_jobs]:
            os.unlink(f)

    def test_sub_7(self):
        assert "PYJOB_ENV1" not in os.environ
        os.environ["PYJOB_ENV1"] = "pyjob_random1"
        jobs = [make_script(["echo $PYJOB_ENV1"], directory=os.getcwd()) for _ in range(2)]
        array_script, array_jobs = prep_array_script(jobs, os.getcwd(), SunGridEngine.TASK_ID)
        jobid = SunGridEngine.sub(array_script, array=[1, 2], directory=os.getcwd(), log=os.devnull,
                                  name=inspect.stack()[0][3], shell="/bin/sh")
        while SunGridEngine.stat(jobid):
            time.sleep(1)
        for i, j in enumerate(jobs):
            f = j.replace(".sh", ".log")
            self.assertTrue(os.path.isfile(f))
            self.assertEqual(os.environ["PYJOB_ENV1"], open(f).read().strip())
            os.unlink(f)
        for f in jobs + [array_script, array_jobs]:
            os.unlink(f)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Testing facility for .local"""

__author__ = "Felix Simkovic"
__date__ = "03 Jun 2017"

import os
import time
import unittest

from pyjob.misc import make_python_script
from pyjob.platform.local import LocalJobServer, SERVER_INDEX 


class TestLocalJobServer(unittest.TestCase):

    def test_sub_1(self):
        j = make_python_script([
            ["import sys, time"], 
            ["print(\"I am job: 1\")"],
            ["sys.exit(0)"],
        ], prefix="unittest")
        l = j.rsplit('.', 1)[0] + '.log'
        LocalJobServer.sub([j], nproc=1)
        time.sleep(0.5)
        self.assertTrue(os.path.isfile(l))
        for f in [j, l]: os.unlink(f)

    def test_sub_2(self):
        jobs = [
            make_python_script([
                ["import sys, time"], 
                ["print(\"I am job: {0}\")".format(i)],
                ["sys.exit(0)"],
            ], prefix="unittest") for i in range(6)
        ]
        logs = [j.rsplit('.', 1)[0] + '.log' for j in jobs]
        LocalJobServer.sub(jobs, nproc=2)
        time.sleep(0.5)
        self.assertTrue(os.path.isfile(logs[-1]))
        for f in jobs + logs: os.unlink(f)
    
    def test_kill_1(self):
        jobs = [
            make_python_script([
                ["import sys, time"], 
                ["time.sleep(100)"],
                ["sys.exit(0)"],
            ], prefix="unittest") for i in range(6)
        ]
        logs = [j.rsplit('.', 1)[0] + '.log' for j in jobs]
        jobid = LocalJobServer.sub(jobs, nproc=2)
        time.sleep(0.5)
        self.assertTrue(jobid in SERVER_INDEX)
        LocalJobServer.kill(jobid)
        self.assertFalse(jobid in SERVER_INDEX)
        for l in logs:
            self.assertFalse(os.path.isfile(l))
        for f in jobs: os.unlink(f)
     

if __name__ == "__main__":
    unittest.main(verbosity=2)




# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Job container for automated launch"""

__author__ = "Felix Simkovic"
__date__ = "05 Jun 2017"
__version__ = "0.1"

import logging
import os
import time

from pyjob.exception import PyJobError
from pyjob.misc import is_script
from pyjob.platform import platform_factory, prep_array_script

logger = logging.getLogger(__name__)


class Job(object):
    """Generic :obj:`Job` class to allow for job control

    This class provides full access to various job submission
    platforms with a unified interface.

    """
    __slots__ = ["_lock", "_log", "_pid", "_platform", "_qtype", "_script"]

    def __init__(self, qtype):
        """Instantiate a new :obj:`Job` submission class

        Parameters
        ----------
        qtype : str
           The queue type to submit the jobs to [ local | lsf | sge ]

        Raises
        ------
        ValueError
           Unknown platform

        """
        self._lock = False
        self._pid = None
        self._log = []
        self._platform = None
        self._qtype = None
        self._script = []

        # Check immediately if we have a known platform
        self._platform = platform_factory(qtype)
        self._qtype = qtype

    def __repr__(self):
        """Representation of the :obj:`Job`"""
        return "{0}(pid={1} qtype={2})".format(
            self.__class__.__name__, self.pid, self.qtype
        )

    @property
    def finished(self):
        """Return whether the job has finished"""
        return not bool(self.stat())

    @property
    def log(self):
        """Return a list of the log file(s)"""
        return self._log

    @property
    def pid(self):
        """Return the process id of this job"""
        return self._pid

    @property
    def qtype(self):
        """Return the platform type we assigned to this job"""
        return self._qtype

    @property
    def script(self):
        """Return a list of the script file(s)"""
        return self._script

    def alter(self, priority=None):
        """Alter the job parameters

        Parameters
        ----------
        priority : int, optional
           The priority level of the job

        """
        return self._platform.alt(self.pid, priority=priority)

    def hold(self):
        """Hold the job"""
        return self._platform.hold(self.pid)

    def kill(self):
        """Kill the job"""
        return self._platform.kill(self.pid)

    def release(self):
        """Release the job"""
        return self._platform.rls(self.pid)

    def submit(self, script, *args, **kwargs):
        """Submit a job to the job management platform

        Parameters
        ----------
        script : str, list
           A list of one or more scripts with absolute paths

        """
        # Only allow one submission per job
        if self._lock:
            logger.debug("This Job instance is locked, for further submissions create a new")
            return

        # Define a directory if not already done
        if not('directory' in kwargs and kwargs['directory']):
            kwargs['directory'] = os.getcwd()

        # Quick check if all scripts are sound - Also keep copy of logs and scripts
        self._script, self._log = Job.check_script(script)

        # See if we need to prepare an array
        if len(self._script) > 1:
            if any(base.__name__ == "ClusterPlatform" for base in self._platform.__bases__):
                script, _ = prep_array_script(self._script, kwargs['directory'], self._platform.TASK_ID)
                kwargs["array"] = [1, len(self._script),
                                   kwargs['max_array_jobs'] if 'max_array_jobs' in kwargs else len(self._script)]
                kwargs["shell"] = "/bin/sh"
                kwargs["log"] = os.devnull

                script = [script]
            else:
                script = self._script
        else:
            script = self._script

        # Get the submission function and submit the job
        self._pid = self._platform.sub(script, **kwargs)
        # Lock this Job so we cannot submit another
        self._lock = True

    def stat(self):
        """Get some data for the job"""
        return self._platform.stat(self.pid)

    def wait(self, check_success=None, interval=30, monitor=None):
        """Wait until all processing has finished

        Parameters
        ----------
        check_success : func, optional
           A function handler to be called to check the success status of a job.
           If the function returns True, the execution will be terminated.

           !!! The function is required to take a single argument, a log file !!!

        interval : int, optional
           The interval to wait between checking (in seconds) [default: 30]
        monitor : func, optional
           A function handler to be called to update, e.g. GUIs

        """
        do_check_success = bool(check_success and callable(check_success))
        if do_check_success:
            logger.debug("Checking for Job %d success with function %s", self.pid, check_success.__name__)
        do_monitor = bool(monitor and callable(monitor))

        while not self.finished:
            # Allow for early termination
            if do_check_success:
                for log in self.log:
                    if os.path.isfile(log) and check_success(log):
                        logger.debug("Job %d succeeded, run log: %s", self.pid, log)
                        self.kill()
            # Allow for GUI updating
            if do_monitor:
                monitor()
            # Wait if nothing else
            time.sleep(interval)

    @staticmethod
    def check_script(script):
        """Check if all scripts are sound"""
        if isinstance(script, str) and is_script(script):
            logs = [script.rsplit('.', 1)[0] + '.log']
            scripts = [script]
        elif (isinstance(script, list) or isinstance(script, tuple)) and all(is_script(fpath) for fpath in script):
            logs = [s.rsplit('.', 1)[0] + '.log' for s in script]
            scripts = list(script)
        else:
            raise PyJobError("One or more scripts cannot be found or are not executable")
        return scripts, logs



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Testing facility for pyjob"""

__author__ = "Felix Simkovic"
__date__ = "10 May 2017"

import os
import sys
import unittest

from pyjob import cexec


class Test(unittest.TestCase):

    def test_cexec_1(self):
        cmd = [sys.executable, "-c", "import sys; print(\"hello\"); sys.exit(0)"]
        stdout = cexec(cmd)
        self.assertEqual("hello", stdout)
    
    def test_cexec_2(self):
        cmd = [sys.executable, "-c", "import sys; sys.exit(1)"]
        self.assertRaises(RuntimeError, cexec, cmd)

    def test_cexec_3(self):
        cmd = [sys.executable, "-c", "import sys; print(\"hello\"); sys.exit(0)"]
        stdout = cexec(cmd, permit_nonzero=True)
        self.assertEqual("hello", stdout)
    
    def test_cexec_4(self):
        if sys.version_info < (3, 0):
            cmd = [sys.executable, "-c", "import sys; print(raw_input()); sys.exit(0)"]
        else:
            cmd = [sys.executable, "-c", "import sys; print(input()); sys.exit(0)"]
        stdout = cexec(cmd, stdin="hello")
        self.assertEqual("hello", stdout)

    def test_cexec_5(self):
        cmd = [sys.executable, "-c", "import os, sys; print(os.getcwd()); sys.exit(0)"]
        directory = os.path.join(os.getcwd(), 'pyjob')
        stdout = cexec(cmd, directory=directory)
        self.assertEqual(directory, stdout)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Various functions"""

__author__ = "Felix Simkovic"
__date__ = "03 Jun 2017"
__version__ = "0.1"

import os
import tempfile

from pyjob.platform import SCRIPT_EXT, SCRIPT_HEADER


def StopWatch():
    """Import the :obj:`StopWatch <pyjob.misc.stopwatch.StopWatch>`"""
    from pyjob.misc.stopwatch import StopWatch
    return StopWatch()


def is_script(f):
    """Confirm if a script file is executable"""
    return os.path.isfile(f) and os.access(f, os.X_OK)


def make_script(cmd, directory=None, prefix="tmp", stem=None, suffix=SCRIPT_EXT):
    """Create an executable script
    
    Parameters
    ----------
    cmd : list
       The command to be written to the script. This can be a 1-dimensional 
       or 2-dimensional list, depending on the commands to run.
    directory : str, optional
       The directory to create the script in
    prefix : str, optional
       The script prefix [default: None]
    stem : str, optional
       The steam part of the script name
    suffix : str, optional
       The script suffix [default: POSIX - ``.sh``, Windows - ``.bat``]
    
    Returns
    -------
    str
       The path to the script

    """
    # Get the script name
    script = tmp_file(delete=True, directory=directory, prefix=prefix, stem=stem, suffix=suffix)
    # Write the contents to the file
    with open(script, 'w') as f_out:
        content = SCRIPT_HEADER + os.linesep
        if isinstance(cmd, list) and isinstance(cmd[0], list):
            for c in cmd:
                content += ' '.join(map(str, c)) + os.linesep
        elif isinstance(cmd, list):
            content += ' '.join(map(str, cmd)) + os.linesep
        f_out.write(content)
    os.chmod(script, 0o777)
    return script


def make_python_script(cmd, directory=None, prefix="tmp", stem=None, suffix='.py'):
    """Create an executable Python script

    Parameters
    ----------
    cmd : list
       The command to be written to the script. This can be a 1-dimensional 
       or 2-dimensional list, depending on the Python commands to run.
    directory : str, optional
       The directory to create the script in
    prefix : str, optional
       The script prefix [default: None]
    stem : str, optional
       The steam part of the script name
    suffix : str, optional
       The script suffix [default: ``.py``]

    Returns
    -------
    str
       The path to the script

    """
    # Get the script name
    script = tmp_file(delete=True, directory=directory, prefix=prefix, stem=stem, suffix=suffix)
    # Write the contents to the file
    with open(script, 'w') as f_out:
        content = "#!/usr/bin/env python" + os.linesep
        if isinstance(cmd, list) and isinstance(cmd[0], list):
            for c in cmd:
                content += ' '.join(map(str, c)) + os.linesep
        elif isinstance(cmd, list):
            content += ' '.join(map(str, cmd)) + os.linesep
        f_out.write(content)
    os.chmod(script, 0o777)
    return script


def tmp_dir(directory=None, prefix="tmp", suffix=""):
    """Return a filename for a temporary directory 

    Parameters
    ----------
    directory : str, optional
       Path to a directory to write the files to.
    prefix : str, optional
       A prefix to the temporary filename
    suffix : str, optional
       A suffix to the temporary filename

    """
    return tempfile.mkdtemp(dir=directory, prefix=prefix, suffix=suffix)


def tmp_file(delete=False, directory=None, prefix="tmp", stem=None, suffix=""):
    """Return a filename for a temporary file

    The naming convention of scripts will be ``prefix`` + ``stem`` + ``suffix``.

    Parameters
    ----------
    delete : bool, optional
       Delete the file, thus return name only [default: True]
    directory : str, optional
       Path to a directory to write the files to
    prefix : str, optional
       A prefix to the temporary filename
    stem : str, optional
       The steam part of the script name
    suffix : str, optional
       A suffix to the temporary filename

    """
    if directory is None:
        directory = tempfile.gettempdir()
    if stem is None:
        tmpf = tempfile.NamedTemporaryFile(delete=delete, dir=directory, prefix=prefix, suffix=suffix)
        tmpf.close()
        return tmpf.name
    else:
        tmpf = os.path.join(directory, prefix + stem + suffix)
        if not delete:
            open(tmpf, 'w').close()
        return tmpf




# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""PyJob library for Python-controlled job execution across multiple platforms"""

import logging
import os
import signal
import subprocess
import sys

from pyjob import version
from pyjob.exception import PyJobUnknownFormat

__author__ = "Felix Simkovic"
__email__ = "felixsimkovic@me.com"
__version__ = version.__version__


def Job(*args, **kwargs):
    from pyjob.job import Job
    return Job(*args, **kwargs)


def cexec(cmd, directory=None, stdin=None, permit_nonzero=False):
    """Execute a command

    Parameters
    ----------
    cmd : list
       The command to call
    directory : str, optional
       The directory to execute the job in
    stdin : str, optional
       Additional keywords provided to the command
    permit_nonzero : bool, optional
       Allow non-zero return codes [default: False]

    Returns
    -------
    str
       The processes standard out

    Raises
    ------
    RuntimeError
       Execution exited with non-zero return code

    """
    logger = logging.getLogger(__name__)
    try:
        logger.debug("Executing '%s'", " ".join(cmd))
        kwargs = {"bufsize": 0, "shell": "False"} if os.name == "nt" else {}
        p = subprocess.Popen(cmd, cwd=directory, stdin=subprocess.PIPE, stderr=subprocess.STDOUT,
                             stdout=subprocess.PIPE, **kwargs)
        # We require the str.encode() and str.decode() functions for Python 2.x and 3.x compatibility
        stdout, _ = p.communicate(input=stdin.encode()) if stdin else p.communicate()
        stdout = stdout.decode()
        if p.returncode == 0:
            return stdout.strip()
        elif permit_nonzero:
            logger.debug("Ignoring non-zero returncode %d for '%s'", p.returncode, " ".join(cmd))
            return stdout.strip()
        else:
            msg = "Execution of '{0}' exited with non-zero return code ({1}): {2}".format(' '.join(cmd),
                                                                                          p.returncode, stdout)
            raise RuntimeError(msg)
    # Allow ctrl-c's
    except KeyboardInterrupt:
        os.kill(p.pid, signal.SIGTERM)
        sys.exit(signal.SIGTERM)




# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Testing facility for pyjob.misc.stopwatch"""

__author__ = "Felix Simkovic"
__date__ = "02 Jun 2017"

import time
import unittest

from pyjob.misc.stopwatch import StopWatch


class TestStopWatch(unittest.TestCase):

    def test_start_1(self):
        t = StopWatch()
        self.assertEqual(int(round(t.time)), 0)

    def test_start_2(self):
        t = StopWatch()
        t.start()
        self.assertTrue(t.running)
        time.sleep(1)
        self.assertEqual(int(round(t.time)), 1)

    def test_stop_1(self):
        t = StopWatch()
        t.start()
        time.sleep(1)
        t.stop()
        self.assertFalse(t.running)
        self.assertEqual(int(round(t.time)), 1)

    def test_stop_2(self):
        t = StopWatch()
        t.start()
        time.sleep(1)   # <- count
        t.stop()
        self.assertEqual(int(round(t.time)), 1)
        time.sleep(1)   # <- ignore
        t.start()
        time.sleep(2)   # <- count
        t.stop()
        self.assertEqual(int(round(t.time)), 3)
        self.assertEqual(t.nintervals, 2)

    def test_stop_3(self):
        t = StopWatch()
        t.start()
        time.sleep(1)  # <- count
        t.stop()
        time.sleep(1)  # <- ignore
        t.start()
        time.sleep(2)  # <- count
        t.stop()
        time.sleep(1)  # <- ignore
        t.start()
        time.sleep(1)  # <- count
        t.stop()
        self.assertEqual(int(round(t.time)), 4)
        self.assertEqual(t.nintervals, 3)

    def test_lap_1(self):
        t = StopWatch()
        self.assertEqual(t.lap, None)

    def test_lap_2(self):
        t = StopWatch()
        t.start()
        time.sleep(1)
        self.assertEqual(int(round(t.lap.time)), 1)
        self.assertEqual(t[0].nlaps, 1)

    def test_lap_3(self):
        t = StopWatch()
        t.start()
        time.sleep(1)
        self.assertEqual(int(round(t.lap.time)), 1)
        time.sleep(3)
        self.assertEqual(int(round(t.lap.time)), 3)
        self.assertEqual(t[0].nlaps, 2)

    def test_lap_4(self):
        t = StopWatch()
        t.start()
        time.sleep(1)  # <- count
        self.assertEqual(int(round(t.lap.time)), 1)
        time.sleep(3)  # <- count
        self.assertEqual(int(round(t.lap.time)), 3)
        t.stop()
        time.sleep(1)  # <- ignore
        t.start()
        time.sleep(2)  # <- count
        self.assertEqual(int(round(t.lap.time)), 2)
        t.stop()
        self.assertEqual(t.nintervals, 2)
        self.assertEqual(t[0].nlaps, 2)
        self.assertEqual(t[1].nlaps, 1)

    def test_reset_1(self):
        t = StopWatch()
        t.start()
        time.sleep(2)
        t.stop()
        self.assertEqual(int(round(t.time)), 2)
        t.reset()
        self.assertEqual(int(round(t.time)), 0)

    def test_runtime_1(self):
        t = StopWatch()
        t.start()
        time.sleep(2)
        t.stop()
        self.assertEqual(int(round(t.time)), 2)

    def test_runtime_2(self):
        t = StopWatch()
        t.start()
        time.sleep(2)
        self.assertEqual(int(round(t.time)), 2)

    def test_runtime_3(self):
        t = StopWatch()
        t.start()
        time.sleep(2)
        self.assertEqual(int(round(t.time)), 2)
        time.sleep(2)
        self.assertEqual(int(round(t.time)), 4)

    def test_runtime_4(self):
        t = StopWatch()
        t.start()
        time.sleep(2)
        self.assertEqual(int(round(t.time)), 2)
        time.sleep(1)
        self.assertEqual(int(round(t.time)), 3)
        time.sleep(1)
        self.assertEqual(int(round(t.time)), 4)

    def test_runtime_5(self):
        t = StopWatch()
        t.start()
        time.sleep(2)   # <- count
        time.sleep(1)   # <- count
        t.stop()
        time.sleep(2)   # <- ignore
        t.start()
        time.sleep(1)   # <- count
        t.stop()
        self.assertEqual(int(round(t.time)), 4)

    def test_convert_1(self):
        t = StopWatch()
        t.start()
        time.sleep(2)
        t.stop()
        self.assertEqual((0, 0, 0, 2), t.time_pretty)

    def test_convert_2(self):
        t = StopWatch()
        t.start()
        time.sleep(5)
        t.stop()
        self.assertEqual((0, 0, 0, 5), t.time_pretty)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module to store local job management code"""

__author__ = "Felix Simkovic"
__date__ = "09 May 2017"
__version__ = "0.1"

import collections
import logging
import multiprocessing
import random
import time

from pyjob import cexec
from pyjob.platform.platform import LocalPlatform

logger = logging.getLogger(__name__)


class _Worker(multiprocessing.Process):
    """Simple manual worker class to execute jobs in the queue"""

    def __init__(self, queue, directory=None, permit_nonzero=False):
        """Instantiate a new worker

        Parameters
        ----------
        queue : obj
           An instance of a :obj:`Queue <multiprocessing.Queue>`
        directory : str, optional
           The directory to execute the jobs in
        permit_nonzero : bool, optional
           Allow non-zero return codes [default: False]

        """
        super(_Worker, self).__init__()
        self.directory = directory
        self.permit_nonzero = permit_nonzero
        self.queue = queue

    def run(self):
        """Method representing the process's activity"""
        for job in iter(self.queue.get, None):
            stdout = cexec([job], directory=self.directory, permit_nonzero=self.permit_nonzero)
            with open(job.rsplit('.', 1)[0] + '.log', 'w') as f_out:
                f_out.write(stdout)
 

# Store a reference to the Workers
SERVER_INDEX = collections.defaultdict(list)


class LocalJobServer(LocalPlatform):
    """A local server to execute jobs via the multiprocessing module
    
    Examples
    --------

    The most basic example of a :obj:`LocalJobServer` is to run scripts across one or
    more processors on a local machine. This can be achieved with the following example.

    >>> from pyjob.misc import make_python_script
    >>> from pyjob.platform import LocalJobServer
    >>> scripts = [
    ...     make_python_script(["import sys;", "print('hello');", "sys.exit(0);"])
    ...     for _ in range(3)
    ... ]
    >>> LocalJobServer.sub(scripts, nproc=2)

    This will create three Python script files and execute them by calling :func:`sub <LocalJobServer.sub>`.
    
    Sometimes you might want to submit many jobs where you know that some are going to fail. In this 
    case, you can also use the :obj:`LocalJobServer` and provide the ``permit_nonzero`` keyword argument,
    which will allow non-zero return codes from commands.

    """

    @staticmethod
    def kill(jobid):
        """Remove a job from the local process list
        
        Parameters
        ----------
        jobid : int
           The job id to remove
        
        """
        if jobid in SERVER_INDEX:
            for wk in SERVER_INDEX[jobid]:
                if wk.is_alive():
                    wk.terminate()
            logger.debug("Terminated job %d", jobid)
            SERVER_INDEX.pop(jobid)
        else:
            logger.debug("Job %d not in queue", jobid)

    @staticmethod
    def stat(jobid):
        """Obtain information about a job id
        
        Parameters
        ----------
        jobid : int
           The job id to remove
        
        """
        if SERVER_INDEX and any(wk.is_alive() for wk in SERVER_INDEX[jobid]):
            return {'job_number': jobid, 'status': "Running"}
        else:
            return {}

    @staticmethod
    def sub(command, directory=None, nproc=1, permit_nonzero=False, *args, **kwargs):
        """Submission function for local job submission via ``multiprocessing``
        
        Parameters
        ----------
        command : list
           A list with the final command
        directory : str, optional
           The directory to execute the jobs in
        nproc : int, optional
           The number of processors to use
        permit_nonzero : bool, optional
           Allow non-zero return codes [default: False]
        runtime : int, optional
           The maximum runtime of the job in seconds

        """
        # Create a new queue
        queue = multiprocessing.Queue()
        
        # Create workers equivalent to the number of jobs
        workers = []
        for _ in range(nproc):
            wp = _Worker(queue, directory=directory, permit_nonzero=permit_nonzero)
            wp.start()
            workers.append(wp)
        # Add each command to the queue
        for cmd in command:
            queue.put(cmd)
        # Stop workers from exiting without completion
        for _ in range(nproc):
            queue.put(None)
        # Disallow addition of further jobs
        queue.close()
        # Need this in case we kill the job immediately after submitting
        time.sleep(0.1)
        # Save these workers
        while True:
            jobid = random.randint(1, 1000) 
            if jobid in SERVER_INDEX:
                continue
            else:
                break
        SERVER_INDEX[jobid] = workers
        return jobid



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Testing facility for pyjob.job"""

__author__ = "Felix Simkovic"
__date__ = "06 May 2017"

import glob
import os
import unittest

from pyjob.exception import PyJobError
from pyjob.job import Job
from pyjob.misc import SCRIPT_EXT, make_script, tmp_file

# Only need to check two, one for cluster and local
platform = "sge" if "SGE_ROOT" in os.environ else "local"


class TestJob(unittest.TestCase):

    def test_submit_1(self):
        ss = make_script(["sleep 1"])
        ls = ss.replace(SCRIPT_EXT, ".log")
        j = Job(platform)
        j.submit(ss, nproc=2, log=ls)
        j.wait()
        for f in [ss, ls]:
            os.unlink(f)

    def test_submit_2(self):
        ss = [make_script(["sleep 1"])]
        ls = [ss[0].replace(SCRIPT_EXT, ".log")]
        j = Job(platform)
        j.submit(ss, nproc=2, log=ls[0])
        j.wait()
        for f in ss + ls:
            os.unlink(f)

    def test_submit_3(self):
        ss = [make_script(["sleep 1"]) for _ in range(5)]
        ls = [f.replace(SCRIPT_EXT, ".log") for f in ss]
        j = Job(platform)
        j.submit(ss, nproc=2, log=os.devnull)
        j.wait()
        for f in ss + ls + glob.glob("array_*"):
            os.unlink(f)

    def test_check_script_1(self):
        ss = [make_script(["sleep 1"])]
        ls = [f.replace(SCRIPT_EXT, ".log") for f in ss]
        s, l = Job.check_script(ss)
        self.assertEqual(ss, s)
        self.assertEqual(ls, l)
        for f in ss:
            os.unlink(f)

    def test_check_script_2(self):
        ss = [make_script(["sleep 1"]) for _ in range(5)]
        ls = [f.replace(SCRIPT_EXT, ".log") for f in ss]
        s, l = Job.check_script(ss)
        self.assertEqual(ss, s)
        self.assertEqual(ls, l)
        for f in ss:
            os.unlink(f)

    def test_check_script_3(self):
        ss = make_script(["sleep 1"])
        ls = ss.replace(SCRIPT_EXT, ".log")
        s, l = Job.check_script(ss)
        self.assertEqual([ss], s)
        self.assertEqual([ls], l)
        os.unlink(ss)

    def test_check_script_4(self):
        ss = tmp_file(delete=False)
        with open(ss, "w") as f_out:
            f_out.write("sleep 1")
        self.assertRaises(PyJobError, Job.check_script, ss)


if __name__ == "__main__":
    unittest.main(verbosity=2)



"""Job Management Controller"""

__author__ = "Felix Simkovic"
__version__ = "1.0"

from distutils.util import convert_path
try:
    from setuptools import setup
except ImportError:
    import sys
    msg("Please install setuptools first!")
    sys.exit(msg)

import os

# ==============================================================
# Functions, functions, functions ... 
# ==============================================================


def readme():
    with open('README.rst', 'r') as f_in:
        return f_in.read()


def version():
    """Get the current PyJob version"""
    # Credits to http://stackoverflow.com/a/24517154
    main_ns = {}
    ver_path = convert_path(os.path.join('pyjob', 'version.py'))
    with open(ver_path) as f_in:
        exec(f_in.read(), main_ns)
    return main_ns['__version__']


# ==============================================================
# Define all the relevant options
# ==============================================================
AUTHOR = "Felix Simkovic"
AUTHOR_EMAIL = "felixsimkovic@me.com"
DESCRIPTION = __doc__.replace("\n", "")
LICENSE = "MIT License"
LONG_DESCRIPTION = readme()
PACKAGE_DIR = "pyjob"
PACKAGE_NAME = "pyjob"
URL = "https://github.com/fsimkovic/pyjob"
VERSION = version()

PACKAGES = [
    'pyjob',
    'pyjob/misc',
    'pyjob/platform',
]

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Science/Research",
    "License :: OSI Approved :: BSD License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
]

# Do the actual setup below
setup(
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    name=PACKAGE_NAME,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    license=LICENSE,
    version=VERSION,
    url=URL,
    packages=PACKAGES,
    package_dir={PACKAGE_NAME: PACKAGE_DIR},
    classifiers=CLASSIFIERS,
    test_suite='nose.collector',
    tests_require=['nose >=1.3.7'],
    zip_safe=False,
)



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Testing facility for pyjob.misc.__init__"""

__author__ = "Felix Simkovic"
__date__ = "29 May 2017"

import os
import shutil
import unittest

from pyjob.misc import tmp_dir, tmp_file


class Test(unittest.TestCase):

    def test_tmp_dir_1(self):
        tmp = tmp_dir()
        self.assertTrue(os.path.isdir(tmp))
        shutil.rmtree(tmp)

    def test_tmp_dir_2(self):
        tmp = tmp_dir(directory=os.getcwd())
        self.assertTrue(os.path.isdir(tmp))
        path, _ = os.path.split(tmp)
        self.assertEqual(os.getcwd(), path)
        shutil.rmtree(tmp)

    def test_tmp_dir_3(self):
        tmp = tmp_dir(prefix="first")
        self.assertTrue(os.path.isdir(tmp))
        _, name = os.path.split(tmp)
        self.assertTrue(name.startswith("first"))
        shutil.rmtree(tmp)

    def test_tmp_dir_4(self):
        tmp = tmp_dir(suffix="last")
        self.assertTrue(os.path.isdir(tmp))
        _, name = os.path.split(tmp)
        self.assertTrue(name.endswith("last"))
        shutil.rmtree(tmp)

    def test_tmp_dir_5(self):
        tmp = tmp_dir(suffix="simbad.the-last")
        self.assertTrue(os.path.isdir(tmp))
        _, name = os.path.split(tmp)
        self.assertTrue(name.endswith("simbad.the-last"))
        shutil.rmtree(tmp)

    def test_tmp_dir_6(self):
        tmp = tmp_dir(directory=os.getcwd(), prefix="first", suffix="last")
        self.assertTrue(os.path.isdir(tmp))
        path, name = os.path.split(tmp)
        self.assertEqual(os.getcwd(), path) 
        self.assertTrue(name.startswith("first"))
        self.assertTrue(name.endswith("last"))
        shutil.rmtree(tmp)

    def test_tmp_file_1(self):
        tmp = tmp_file()
        self.assertTrue(os.path.isfile(tmp))
        os.remove(tmp)

    def test_tmp_file_2(self):
        tmp = tmp_file(delete=True)
        self.assertFalse(os.path.isfile(tmp))

    def test_tmp_file_3(self):
        tmp = tmp_file(directory=os.getcwd())
        self.assertTrue(os.path.isfile(tmp))
        path, _ = os.path.split(tmp)
        self.assertEqual(os.getcwd(), path)
        os.remove(tmp)

    def test_tmp_file_4(self):
        tmp = tmp_file(prefix="first")
        self.assertTrue(os.path.isfile(tmp))
        self.assertTrue(os.path.basename(tmp).startswith("first"))
        os.remove(tmp)

    def test_tmp_file_5(self):
        tmp = tmp_file(stem="middle")
        self.assertTrue(os.path.isfile(tmp))
        self.assertTrue("middle" in os.path.basename(tmp))
        os.remove(tmp)

    def test_tmp_file_6(self):
        tmp = tmp_file(suffix="last")
        self.assertTrue(os.path.isfile(tmp))
        self.assertTrue(os.path.basename(tmp).endswith("last"))
        os.remove(tmp)

    def test_tmp_file_7(self):
        tmp = tmp_file(delete=True, stem="middle")
        self.assertFalse(os.path.isfile(tmp))
        self.assertTrue("middle" in os.path.basename(tmp))

    def test_tmp_file_8(self):
        tmp = tmp_file(delete=False, directory=os.getcwd(), prefix="first", stem="middle", suffix="last")
        self.assertTrue(os.path.isfile(tmp))
        path, name = os.path.split(tmp)
        self.assertEqual(os.getcwd(), path)
        self.assertTrue(name.startswith("first"))
        self.assertTrue("middle" in name)
        self.assertTrue(name.endswith("last"))
        os.remove(tmp)
    
    def test_tmp_file_9(self):
        tmp = tmp_file(delete=False, directory=os.getcwd(), prefix="first", suffix="last")
        self.assertTrue(os.path.isfile(tmp))
        path, name = os.path.split(tmp)
        self.assertEqual(os.getcwd(), path)
        self.assertTrue(name.startswith("first"))
        self.assertTrue(name.endswith("last"))
        os.remove(tmp)
    

if __name__ == "__main__":
    unittest.main(verbosity=2)




# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Container for the most basic platform layout"""

__author__ = "Felix Simkovic"
__date__ = "03 Jun 2017"
__version__ = "0.1"

from pyjob.exception import PyJobNotImplemented


class _Platform(object):
    """Container for job management platforms. 
    
    Notes
    -----
    Do not instantiate directly but use child classes instead.
    
    """

    TASK_ID = None

    @staticmethod
    def alt(*args, **kwargs):
        raise PyJobNotImplemented("Function unavailable for specified queue type")

    @staticmethod
    def hold(*args, **kwargs):
        raise PyJobNotImplemented("Function unavailable for specified queue type")

    @staticmethod
    def kill(*args, **kwargs):
        raise PyJobNotImplemented("Function unavailable for specified queue type")

    @staticmethod
    def rls(*args, **kwargs):
        raise PyJobNotImplemented("Function unavailable for specified queue type")

    @staticmethod
    def sub(*args, **kwargs):
        raise PyJobNotImplemented("Function unavailable for specified queue type")

    @staticmethod
    def stat(*args, **kwargs):
        raise PyJobNotImplemented("Function unavailable for specified queue type")


class ClusterPlatform(_Platform):
    """Container for cluster platforms"""
    pass


class LocalPlatform(_Platform):
    """Container for local platforms"""
    pass



# MIT License
#
# Copyright (c) 2017 Felix Simkovic
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Available platforms"""

__author__ = "Felix Simkovic"
__date__ = "03 Jun 2017"
__version__ = "0.1"

import os
import sys
import tempfile

from pyjob.exception import PyJobUnknownPlatform

# OS-dependent script headers and extensions
if sys.platform.startswith('win'):
    EXE_EXT, SCRIPT_HEADER, SCRIPT_EXT = ('.exe', '', '.bat')
else:
    EXE_EXT, SCRIPT_HEADER, SCRIPT_EXT = ('', '#!/bin/bash', '.sh')


# Little factory function to get the right platform without importing all
def platform_factory(qtype):
    """Return the correct platform handler"""
    qtype = qtype.lower()
    if qtype == "local":
        from pyjob.platform.local import LocalJobServer
        return LocalJobServer
    elif qtype == "lsf":
        from pyjob.platform.lsf import LoadSharingFacility
        return LoadSharingFacility
    elif qtype == "sge":
        from pyjob.platform.sge import SunGridEngine
        return SunGridEngine
    else:
        raise PyJobUnknownPlatform("Unknown platform")


def prep_array_script(scripts, directory, task_env):
    """Prepare multiple jobs to be an array

    Parameters
    ----------
    scripts : list
       The scripts to be run as part of the array
    directory : str
       The directory to create the files in
    task_env : str
       The task environment variable

    Returns
    -------
    str
       The array script
    str
       The file listing all jobs

    """
    # Write all jobs into an array.jobs file
    array_jobs = tempfile.NamedTemporaryFile(delete=False, dir=directory, prefix="array_", suffix=".jobs").name
    with open(array_jobs, 'w') as f_out:
        f_out.write(os.linesep.join(scripts) + os.linesep)
    # Create the actual executable script
    array_script = array_jobs.replace(".jobs", ".script")
    with open(array_script, "w") as f_out:
        # Construct the content for the file
        content = "#!/bin/sh" + os.linesep
        content += 'script=$(awk "NR==$' + task_env + '" ' + array_jobs + ')' + os.linesep
        content += "log=$(echo $script | sed 's/.sh/.log/')" + os.linesep
        content += "$script > $log" + os.linesep
        f_out.write(content)
    return array_script, array_jobs
